<?php
/*
Plugin Name: 		Cryptomus - Crypto Payment Gateway
Plugin URI: 		https://doc.cryptomus.com/sdks-and-modules/modules
Description:		Provides a <a href="https://cryptomus.com">Cryptomus.com</a> Crypto Payment Gateway for <a href="https://wordpress.org/plugins/easy-digital-downloads/">Easy Digital Downloads 2.4.2+</a>. Direct Integration on your website, no external payment pages opens (as other payment gateways offer). Accept Bitcoin, BitcoinCash, Litecoin, Dash, Dogecoin, Speedcoin, Reddcoin, Potcoin, Feathercoin, Vertcoin, Peercoin, MonetaryUnit payments online. You will see the bitcoin/altcoin payment statistics in one common table on your website. No Chargebacks, Global, Secure. All in automatic mode. Warning! Need enable permalink.
Version: 			1.0.0
Author: 			Cryptomus.com
Author URI: 		https://cryptomus.com
License: 			GPLv2
License URI: 		http://www.gnu.org/licenses/gpl-2.0.html
*/

use EddCryptomus\PaymentStatus;
use Cryptomus\Api\Client as CryptomusClient;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

require_once plugin_dir_path(__FILE__) . 'vendor/autoload.php';

if (!class_exists('Easy_Digital_Downloads') && !class_exists('Edd_Gateway_Cryptomus')) return;

DEFINE('CRYPTOMUS', "edd-cryptomus");

class Edd_Gateway_Cryptomus
{
    const SETTING_NAME_PAYMENT_KEY = 'cryptomus_payment_key';
    const SETTING_NAME_UUID = 'cryptomus_uuid';
    const SETTING_SUBTRACT = 'subtract';
    const SETTING_LIFETIME = 'lifetime';

    const SETTING_BUTTON_TITLE = 'title';

    public function __construct()
    {
        $this->method_title = __('Cryptomus', CRYPTOMUS);
        $this->method_description = "<a target='_blank' href='https://cryptomus'>" . __('Plugin Homepage', CRYPTOMUS) . "</a><br/>";
        $this->has_fields = false;

        // Gateway Filters
        add_filter('edd_settings_gateways', array($this, 'init_form_fields'));
        add_filter('edd_payment_gateways', array($this, 'register_gateway'));
        add_filter('edd_settings_sections_gateways', array($this, 'register_gateway_section'));
        add_filter('edd_accepted_payment_icons', array($this, 'add_icon'));

        add_action('edd_cryptomus_cc_form', '__return_false');
        add_action('edd_gateway_cryptomus', array($this, 'process_payment'));
        add_action(
            'rest_api_init',
            function () {
                register_rest_route(
                    'cryptomus',
                    'invoice',
                    array(
                        'methods' => 'POST',
                        'permission_callback' => function () {
                            return true;
                        },
                        'callback' => array($this, 'cryptomus_process_webhook'),
                    )
                );
            }
        );

        return true;
    }

    public function register_gateway($gateways)
    {
        $gateways['cryptomus'] = array(
            'admin_label' => 'Cryptomus',
            'checkout_label' => edd_get_option(self::SETTING_BUTTON_TITLE, __('Cryptomus', CRYPTOMUS)),
            'icons' => array(
                plugins_url('./cryptomus.png', __FILE__),
            ),
        );

        return $gateways;
    }

    public function register_gateway_section(array $gateway_sections)
    {
        $gateway_sections['cryptomus'] = __('Cryptomus', CRYPTOMUS);

        return $gateway_sections;
    }

    public function add_icon(array $icons)
    {
        $icon_url = plugins_url('./cryptomus.png', __FILE__);
        $button_title = edd_get_option(self::SETTING_BUTTON_TITLE, __('Cryptomus', CRYPTOMUS));
        $icons[$icon_url] = $button_title;
        return $icons;
    }

    public function init_form_fields($settings)
    {

        $cryptomus_settings = array(
            'cryptomus' => array(
                'id' => 'cryptomus',
                'name' => '<h3>' . __('Cryptomus', CRYPTOMUS) . '</h3><!--' . print_r($settings, true) . ' -->',
                'desc' => "<p>To start using Cryptomus, you need to follow a few simple steps.</p>
        <p>1) Create an account on our website <a href='http://cryptomus.com/?utm_source=wordpress&utm_medium=plugin-description'>cryptomus.com</a> and complete the verification process.</p>
        <p>2) Once you have completed the verification, you will be issued a personal merchant UUID and API key.</p>
        <p>3) In the settings, enter the merchant UUID and API key.</p>
        <p>Congratulations! Now you can accept cryptocurrency payments using Cryptomus.</p>
        <p>Detailed instructions on creating a merchant account can be found <a href='https://cryptomus.com/blog/how-to-accept-crypto-on-your-wordpress-website-with-woocommerce-payment-plugin/?utm_source=wordpress&utm_medium=plugin-description'>here</a>.</p>
        <p>Have questions? We are always happy to answer on <a href='https://t.me/cryptomussupport'>Telegram</a>. Our support operates 24 hours a day.</p>",
                'type' => 'descriptive_text',
            ),
            'payment_key' => array(
                'id' => self::SETTING_NAME_PAYMENT_KEY,
                'name' => __('Payment API-key', CRYPTOMUS),
                'desc' => __('You can find the API key in the settings of your personal account. Read more <a href="https://cryptomus.com/blog/how-to-accept-crypto-on-your-wordpress-website-with-woocommerce-payment-plugin/?utm_source=wordpress&utm_medium=plugin-description">here</a>.', CRYPTOMUS),
                'type' => 'text',
            ),
            'uuid' => array(
                'id' => self::SETTING_NAME_UUID,
                'name' => __('Merchant UUID', CRYPTOMUS),
                'desc' => __('You can find the Merchant UUID in the settings of your personal account. Read more <a href="https://cryptomus.com/blog/how-to-accept-crypto-on-your-wordpress-website-with-woocommerce-payment-plugin/?utm_source=wordpress&utm_medium=plugin-description">here</a>.', CRYPTOMUS),
                'type' => 'text',
            ),
            'subtract' => array(
                'id' => self::SETTING_SUBTRACT,
                'name' => __('How much commission does the client pay (0-100%)', CRYPTOMUS),
                'desc' => __('Percentage of the payment commission charged to the client
                If you have a rate of 1%, then if you create an invoice for 100 USDT with subtract = 100 (the client pays 100"%" commission), the client will have to pay 101 USDT.', CRYPTOMUS),
                'type' => 'number',
                'max'  => 100,
                'min'  => 0,
                'step' => 1,
            ),
            'lifetime' => array(
                'id' => self::SETTING_LIFETIME,
                'name' => __('Lifetime', CRYPTOMUS),
                'desc' => __('The lifespan of the issued invoice', CRYPTOMUS),
                'type' => 'select',
                'options' => array(
                    '3600' => __('1 hour', CRYPTOMUS),
                    '10800' => __('3 hours', CRYPTOMUS),
                    '21600' => __('6 hours', CRYPTOMUS),
                    '43200' => __('12 hours', CRYPTOMUS)
                )
            ),
            'button_title' => array(
                'id' => self::SETTING_BUTTON_TITLE,
                'name' => __('Title', CRYPTOMUS),
                'desc' => __('Button title in card', CRYPTOMUS),
                'type' => 'text'
            ),
        );

        // $gateway_settings = array_merge($settings, $cryptomus_settings);
        $gateway_settings = $settings;
        $gateway_settings['cryptomus'] = $cryptomus_settings;

        return $gateway_settings;
    }

    function cryptomus_process_webhook(WP_REST_Request $request)
    {
        $params = $request->get_params();

        if (empty($params['uuid']) || empty($params['order_id'])) {
            return ['success' => false];
        }

        $payment = $this->getPayment();
        $result = $payment->info(['uuid' => $params['uuid']]);

        if (empty($result['payment_status'])) {
            return ['success' => false];
        }

        $additionalData = json_decode($result['additional_data'], true);
        edd_update_payment_status($additionalData['order_id'], PaymentStatus::convertToWPStatus($result['payment_status']));

        return ['success' => true];
    }

    function process_payment($purchase_data)
    {
        $errors = edd_get_errors();
        $amount = edd_sanitize_amount($purchase_data['price']);

        if (!$errors) {
            $payment_data = array(
                'price' => $amount,
                'date' => $purchase_data['date'],
                'user_email' => $purchase_data['user_email'],
                'purchase_key' => $purchase_data['purchase_key'],
                'currency' => edd_get_currency(),
                'downloads' => $purchase_data['downloads'],
                'cart_details' => $purchase_data['cart_details'],
                'user_info' => $purchase_data['user_info'],
                'status' => 'pending'
            );

            $order_id = edd_insert_payment($payment_data);
            if (!$order_id) {
                edd_set_error('authorize_error', __('Error: Your payment could not be recorded. Please try again.', CRYPTOMUS));
                edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
                return;
            }

            $additionData = ['order_id' => $order_id];
            try {
                $client = $this->getPayment();

                $data = [
                    'amount' => $amount,
                    'currency' => edd_get_currency(),
                    'order_id' => 'edd_' . $order_id,
                    'url_return' => edd_get_success_page_uri() . '?order_id=' . $order_id,
                    'url_callback' => get_site_url(null, "wp-json/cryptomus/invoice"),
                    'is_payment_multiple' => true,
                    'lifetime' => edd_get_option(self::SETTING_LIFETIME, '7200'),
                    'subtract' => edd_get_option(self::SETTING_SUBTRACT, ''),
                    'additional_data' => json_encode($additionData)
                ];

                $paymentCryptomus = $client->create($data);
            } catch (Exception $e) {
                edd_record_gateway_error(__('Cryptomus error', CRYPTOMUS), print_r($e, true), 0);
                edd_set_error('card_declined', __('Error: Cryptomus error create payment ', CRYPTOMUS) . $e->getMessage());
                edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
                return;
            }

            if ($paymentCryptomus['status'] == 'check') {
                $note = __('Cryptomus Invoice ID', CRYPTOMUS) . ': <a target = "_blank" href = "' . $paymentCryptomus['url'] . '">' . $paymentCryptomus['uuid'] . '</a> ' . __('is processing', CRYPTOMUS);
                edd_insert_payment_note($order_id, $note);
                edd_empty_cart();
                wp_redirect($paymentCryptomus['url']);
            } else {
                edd_update_payment_status($order_id, PaymentStatus::convertToWPStatus($paymentCryptomus['payment_status']));
                edd_get_confirmation_page_uri();
            }
        } else {
            edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
        }
    }

    private function getPayment()
    {
        $merchantUuid = trim(edd_get_option(self::SETTING_NAME_UUID, ''));
        $paymentKey = trim(edd_get_option(self::SETTING_NAME_PAYMENT_KEY, ''));

        return CryptomusClient::payment($paymentKey, $merchantUuid);
    }
}

new Edd_Gateway_Cryptomus();
